Module: Axn::Core::SchemaReflection::OutputSchemaMethod
- Defined in:
- lib/axn/core/schema_reflection.rb
Instance Method Summary collapse
-
#output_schema ⇒ Object
See input_schema: validated once per class, over the schema being returned.
Instance Method Details
#output_schema ⇒ Object
See input_schema: validated once per class, over the schema being returned.
73 74 75 |
# File 'lib/axn/core/schema_reflection.rb', line 73 def output_schema Axn::Internal::Reflection::PropertyNames.validated_output(self) { Axn::Internal::Reflection::Schema.build_output(external_field_configs) } end |